#include <osgDB/ReadFile>
#include <osgCal/CoreModel>

using namespace osgCal;

CoreModel::CoreModel() {
}

void CoreModel::loadAllTextures(std::string directory) {
    int materialId;
    for(materialId = 0; materialId < coreModel.getCoreMaterialCount(); materialId++)
    {
	// get the core material
	CalCoreMaterial *pCoreMaterial;
	pCoreMaterial = coreModel.getCoreMaterial(materialId);

	// loop through all maps of the core material
	int mapId;
	for(mapId = 0; mapId < pCoreMaterial->getMapCount(); mapId++)
	{
	    // get the filename of the texture
	    std::string strFilename;
	    strFilename = pCoreMaterial->getMapFilename(mapId);

	    // load the texture from the file
	    osg::Image *img = osgDB::readImageFile(directory+"/"+strFilename);
	    osg::Texture2D *texture = new osg::Texture2D();
	    texture->setImage(img);
	    textures.push_back(texture);

	    // store the opengl texture id in the user data of the map
	    pCoreMaterial->setMapUserData(mapId, (Cal::UserData)texture);
	}
    }
    // make one material thread for each material
    // NOTE: this is not the right way to do it, but this viewer can't do the right
    // mapping without further information on the model etc., so this is the only
    // thing we can do here.
    // NOTE from Ryu: I don't know that the hell is a material thread, so I
    // paste this code as is and pray for it to work...
    for(materialId = 0; materialId < coreModel.getCoreMaterialCount(); materialId++)
    {
	// create the a material thread
	coreModel.createCoreMaterialThread(materialId);

	// initialize the material thread
	coreModel.setCoreMaterialId(materialId, 0, materialId);
    }
}

void CoreModel::loadSkeleton(std::string skeleton_file) {
    if (coreModel.loadCoreSkeleton(skeleton_file)) {
	skeleton=skeleton_file;
    } else {
        CalError::printLastError();
    }
}

void CoreModel::loadAnimation(std::string animation_file) {
    if (coreModel.loadCoreAnimation(animation_file) >= 0) {
	animations.push_back(animation_file);
    } else {
        CalError::printLastError();
    }
}

void CoreModel::loadMesh(std::string mesh_file) {
    if (coreModel.loadCoreMesh(mesh_file) >= 0) {
	meshes.push_back(mesh_file);
    } else {
        CalError::printLastError();
    }
}

void CoreModel::loadMaterial(std::string material_file) {
    if (coreModel.loadCoreMaterial(material_file) >= 0) {
	materials.push_back(material_file);
    } else {
        CalError::printLastError();
    }
}

osg::Object* CoreModel::clone(const osg::CopyOp&) const {
    osgCal::CoreModel *model = new osgCal::CoreModel();
    return model;
}
